/*******************************************************************************
* FILE NAME: Ultrasonic_sensor.c
*
* DESCRIPTION:
*  Handles the Ultrasonic rangefinder processing
*  This was derived from the example sensor testing code posted by VexLabs
*
*  A timer is used to measure the duration of the pulse echo from the sensor.  
*  The data is somewhat unstable at various distances, it uses a simple filter to 
*  throw out irregular data.
*
*  ~44" is the maximum range of the device
*  ~1"  is the minimum range
*  
*  Basic operation is to periodically send the sensor a signal ordering a pulse to
*  be emitted, then listening for the echo while timing how long it takes to return.
*
*  Ultrasonic sensor:
*   Input  - tell the sensor to emit a pulse periodically 
*         (send the sensor a 13us signal every 60us or so)
*   Output - signals the start of the pulse by the raising edge of an interrupt
*         and signals the pulse return echo by the falling edge.
*         Time the duration to calculate the distance (speed of sound at sea level)
*
*  Sensor Input line (trigger a pulse)= rc_dig_out05
*  Sensor Output line (echo response) = rc_dig_int1
*
*  Operation:
*      Enable interrupt (on sensor output line)
*      Send trigger (on sensor input line)
*      Receive interrupt (on sensor output line)
*      Increment HiResCounter
*      Receive interrupt (on sensor output line)
*      SonicDataValid
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "stdio.h"

#define DECLARE_VARIABLES
  #include "Ultrasonic_sensor.h"
#undef DECLARE_VARIABLES

#define TRIGGERPULSE 192   // Create a 13us pulse with Timer 0 (0xC0)

unsigned char triggerCnt;  /* Used to trigger a 13us pulse to Sonic sensor every 50ms */


/*******************************************************************************
* Routine NAME: Filter_Rangefinder
* DESCRIPTION: Filter return echos to eliminate the noise.
*   fltHiResCounter is our final answer to our current distance
*******************************************************************************/
void Filter_Rangefinder(void)
{
  /* Apply a filter to ultrasonic data */
  if (HiResCounter < 50)
  {
    if (fltHiResCounter == HiResCounter) 
    {
      SonicDataValid = 1;
      fltCounter = 0;
      Delta = fltLowResCounter - LowResCounter;
      if (Delta & 0x80)   /* If negative then negate it */
        Delta = !Delta;
      if (Delta > 20)
        fltCounter++;
    }
    else
    {
      fltCounter++;
    }

    if (fltCounter > 3)
    {
       SonicDataValid = 1;
       fltHiResCounter = HiResCounter;
       fltLowResCounter = LowResCounter;
    }
  }
}

/*******************************************************************************
* Routine NAME: Handle_SonicEchoPulse
* DESCRIPTION: Listens for the echo return and times it to get a distance
*******************************************************************************/

void Handle_SonicEchoPulse(void)
{
  if (INTCON2bits.INTEDG2)      /* rising edge ? */
  {                             /* Yes - now start timing the echo*/
    T0CONbits.T0PS0 = 1;        /* Change scale to DIV 8 (204us delay)*/
    TMR0H = TMR0L = 0;          /* reset timer registers */
    INTCONbits.TMR0IF = 0;      /* make sure int flag is clear */
    INTCONbits.TMR0IE = 1;      /* Enable TMR0 Int */
    INTCON2bits.INTEDG2 = 0;    /* trigger of falling edge */
    HiResCounter = LowResCounter = 0;
  }
  else
  {
    INTCON3bits.INT2IE = 0;     /* disable rc_dig_int1 for now */
    LowResCounter = TMR0L;
    INTCONbits.TMR0IE = 0;      /* disable TMR0 Int */

    Filter_Rangefinder();
  }
}


/*******************************************************************************
* Routine NAME: Ultra_Sensor
* DESCRIPTION: This periodically orders a sonic pulse to be sent out
*******************************************************************************/
void Ultra_Sensor(void)
{
  unsigned int pot;

//Each time we receive valid Sonic data & we're at the right distance, light an LED
  if (SonicDataValid)  
  {
    SonicDataValid = 0;
//    printf("H %d, L %d, D %d\r",(int)fltHiResCounter,(int)fltLowResCounter,(int)Delta);

    // Light an LED if the range is as requested by the pot
    // Use a pot to select our distance
    pot  = Get_Analog_Value(POT_INPUT)/10; //Use a range of 0-102
//    printf(" pot=%d, fltHiResCounter=%d\r",(int)pot, (int)fltHiResCounter);

    if ( (fltHiResCounter >= pot) && (fltHiResCounter <= (pot+DISTANCE_TOLERANCE)) )
      ONBOARD_LED = ON;
    else
      ONBOARD_LED = OFF;
  }

//Periodically, issue another sonic pulse
  triggerCnt++;
  if (triggerCnt > 2)         /* expires every 4 * 18.5ms */
  {
    // Enable interrupt to look for the start of a sonic pulse
//    printf(" Start_rc_dig_int1 \r");
    INTCON2bits.INTEDG2 = 1;  /* rc_dig_int1 - Interrupt on rising edge */
    INTCONbits.TMR0IE = 0;    /* make sure TMR0 Int is off */
    T0CONbits.T0PS0 = 0;
    TMR0H = 0;
    INTCONbits.TMR0IF = 0;
    TMR0L = TRIGGERPULSE;
    SONIC_OUTPUT = 1;         /* prepare a 13ms pulse - rising edge of pulse */
    while (!INTCONbits.TMR0IF); //Wait 13ms
    SONIC_OUTPUT = 0;          /* falling edge of pulse */
    triggerCnt = 0;           /* reset trigger counter */
    INTCON3bits.INT2IE = 1;   /* enable rc_dig_int1*/
  }
}


/*******************************************************************************
* Routine NAME: Ultrasonic_Initialization
* DESCRIPTION: Need a timer and an interrupt to operate the ultrasonic sensor.
*******************************************************************************/
void Ultrasonic_Initialization(void)
{
  //Initialize Timer 0 to control the sonic rangefinder
  T0CON = 0;                /* Timer 0 will be used for the sonic sensor */
  T0CONbits.T08BIT = 1;     /* Make it an 8 bit timer */
  T0CONbits.T0PS0  = 0;
  T0CONbits.T0PS1  = 0;
  T0CONbits.T0PS2  = 0;
  T0CONbits.TMR0ON = 1;     /* Turn it on */

  // Initialize the interrupt used to report sonic echos
  INTCONbits.TMR0IE  = 0;   /* disable TMR0 ints for now */
  INTCON2bits.TMR0IP = 0;   /* Make TMR0 low priority */
  INTCON3bits.INT2IE = 0;   /* disable rc_dig_int1 for now */
}
